/***************************************************************************
    begin       : Mon Jul 14 2008
    copyright   : (C) 2008 by Martin Preuss
    email       : martin@libchipcard.de


 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef GWEN_MUTEX_H
#define GWEN_MUTEX_H

#include <gwenhywfar/gwenhywfarapi.h>

#ifdef __cplusplus
extern "C" {
#endif


typedef struct GWEN_MUTEX GWEN_MUTEX;


GWENHYWFAR_API GWEN_MUTEX *GWEN_Mutex_new(void);
GWENHYWFAR_API void GWEN_Mutex_free(GWEN_MUTEX *mtx);

GWENHYWFAR_API int GWEN_Mutex_Lock(GWEN_MUTEX *mtx);
GWENHYWFAR_API int GWEN_Mutex_Unlock(GWEN_MUTEX *mtx);


#ifdef __cplusplus
}
#endif


#endif
